local addon = WakeSpams
local module = addon:NewModule("Options")
WakeSpams.Options = module

local format = format
local UnitName, GetSpellLink, UnitExists, UnitGUID, UnitHealthMax, GetUnitName = UnitName, GetSpellLink, UnitExists, UnitGUID, UnitHealthMax, GetUnitName;
local GetSpellInfo = _G["GetSpellInfo"];
local GetTime = _G["GetTime"];

local ACD = LibStub("AceConfigDialog-3.0")
local ACR = LibStub("AceConfigRegistry-3.0")
local db

local options
local helptext = "spell:name = spellname,\
spell:link = spell link,\
spell:duration = duration,\
spell:target = targetname,\
\
spell:amount = amount (heal, damage, etc),\
spell:extra = resist type for resists, spell names for interrupted spells, etc.\
spell:extra:link = same as above, but with spell links if available."

function module:RegisterOptions()
	-- xD
	local function SetChannelForm(name)
		local arg = ""
		name = addon:Explode("_", name) -- Underscore is OK since channel names can't contain them
		if(name[1] == "CHANNEL") then
			-- Custom channel
			arg = name[2]
			name = name[1]
		end
		name = (name[2]) and name[2] or name
		if(name == "WHISPER") then
			arg = "TARGET"
		end
		arg = {name, arg}
		return arg
	end
	local function GetChannelForm(name)
		local arg = name[2]
		name = name[1]
		if(name == "SELF") 			then return ("!1_%s"):format(name) end
		if(name == "SAY") 			then return ("!2_%s"):format(name) end
		if(name == "YELL") 			then return ("!3_%s"):format(name) end
		if(name == "PARTY") 		then return ("!4_%s"):format(name) end
		if(name == "RAID") 			then return ("!5_%s"):format(name) end
		if(name == "BATTLEGROUND") 	then return ("!6_%s"):format(name) end
		if(name == "WHISPER") 		then return ("!7_%s"):format(name) end
		return ("%s_%s"):format(name,arg)
	end
	local function OutputVals(spellid)
		local valtbl = {
			["!1_SELF"] = "Self",
			["!2_SAY"] = "Say",
			["!3_YELL"] = "|cffff4040Yell|r",
			["!4_PARTY"] = "|cffababffParty|r",
			["!5_RAID"] = "|cffff8000Raid|r",
			["!6_BATTLEGROUND"] = "|cffff3d00Battleground|r",
			["!7_WHISPER"] = "Whisper Target",
			["!8_CHANNEL"] = "------------------------------",
		}
		if(spellid) then
			if db.profile.SpellDB[spellid].aoe then
				valtbl["!7_WHISPER"] = nil
			end
			valtbl["!0_GLOBAL"] = "[GLOBAL]"
		end
		-- Find current custom channels
		local i = 1
		local t
		while(i) do
			local name, _, _, _, _, _, category, _, _ = GetChannelDisplayInfo(i);
			if not name then
				break
			end
			if(category == "CHANNEL_CATEGORY_CUSTOM") then
				valtbl["CHANNEL_"..name] = name
			end
			i = i + 1
		end
		
		return valtbl
	end
	local function SetupOutputOptions(spellid)
		if(spellid) then
			if not db.profile.SpellDB[spellid].OutputFlags then
				db.profile.SpellDB[spellid].OutputFlags = {}
			end
		end
		local tbl = {
			header = {
				name = "Output Options",
				type = "header",
				order = 1,
			},
			solo = {
				name = "When Solo:",
				type = "select",
				order = 2,
				values = OutputVals(spellid),
				style = "dropdown",
				width = "normal",
				set = function(info,val)
					val = SetChannelForm(val) 
					if(spellid) then
						if(val[1] == "GLOBAL") then val = nil end
						db.profile.SpellDB[spellid].OutputFlags[1] = val or nil
					else
						db.profile.OutputFlags[1] = val or nil
					end
				end,
				get = function()
					local v
					if(spellid) then
						if not db.profile.SpellDB[spellid].OutputFlags[1] then
							v = "!0_GLOBAL"
						else
							v = GetChannelForm(db.profile.SpellDB[spellid].OutputFlags[1])
						end
					else
						v = GetChannelForm(db.profile.OutputFlags[1])
					end
					return v
				end,
			},
			party = {
				name = "When in a Party:",
				type = "select",
				order = 3,
				values = OutputVals(spellid),
				style = "dropdown",
				width = "normal",
				set = function(info,val)
					val = SetChannelForm(val) 
					if(spellid) then
						if(val[2] == "GLOBAL") then val = nil end
						db.profile.SpellDB[spellid].OutputFlags[2] = val or nil
					else
						db.profile.OutputFlags[2] = val or nil
					end
				end,
				get = function()
					local v
					if(spellid) then
						if not db.profile.SpellDB[spellid].OutputFlags[2] then
							v = "!0_GLOBAL"
						else
							v = GetChannelForm(db.profile.SpellDB[spellid].OutputFlags[2])
						end
					else
						v = GetChannelForm(db.profile.OutputFlags[2])
					end
					return v
				end,
			},
			raid = {
				name = "When in a Raid:",
				type = "select",
				order = 4,
				values = OutputVals(spellid),
				style = "dropdown",
				width = "normal",
				set = function(info,val)
					val = SetChannelForm(val) 
					if(spellid) then
						if(val[3] == "GLOBAL") then val = nil end
						db.profile.SpellDB[spellid].OutputFlags[3] = val or nil
					else
						db.profile.OutputFlags[3] = val or nil
					end
				end,
				get = function()
					local v
					if(spellid) then
						if not db.profile.SpellDB[spellid].OutputFlags[3] then
							v = "!0_GLOBAL"
						else
							v = GetChannelForm(db.profile.SpellDB[spellid].OutputFlags[3])
						end
					else
						v = GetChannelForm(db.profile.OutputFlags[3])
					end
					return v
				end,
			},
			bg = {
				name = "When in a Battleground:",
				type = "select",
				order = 6,
				values = OutputVals(spellid),
				style = "dropdown",
				width = "normal",
				set = function(info,val)
					val = SetChannelForm(val) 
					if(spellid) then
						if(val[4] == "GLOBAL") then val = nil end
						db.profile.SpellDB[spellid].OutputFlags[4] = val or nil
					else
						db.profile.OutputFlags[4] = val or nil
					end
				end,
				get = function()
					local v
					if(spellid) then
						if not db.profile.SpellDB[spellid].OutputFlags[4] then
							v = "!0_GLOBAL"
						else
							v = GetChannelForm(db.profile.SpellDB[spellid].OutputFlags[4])
						end
					else
						v = GetChannelForm(db.profile.OutputFlags[4])
					end
					return v
				end,
			},
			wg = {
				name = "When in Wintergrasp or Tol Barad:",
				type = "select",
				order = 7,
				values = OutputVals(spellid),
				style = "dropdown",
				width = "normal",
				set = function(info,val)
					val = SetChannelForm(val) 
					if(spellid) then
						if(val[5] == "GLOBAL") then val = nil end
						db.profile.SpellDB[spellid].OutputFlags[5] = val or nil
					else
						db.profile.OutputFlags[5] = val or nil
					end
				end,
				get = function()
					local v
					if(spellid) then
						if not db.profile.SpellDB[spellid].OutputFlags[5] then
							v = "!0_GLOBAL"
						else
							v = GetChannelForm(db.profile.SpellDB[spellid].OutputFlags[5])
						end
					else
						v = GetChannelForm(db.profile.OutputFlags[5])
					end
					return v
				end,
			},
			arena = {
				name = "When in Arena:",
				type = "select",
				order = 5,
				values = OutputVals(spellid),
				style = "dropdown",
				width = "normal",
				set = function(info,val)
					val = SetChannelForm(val) 
					if(spellid) then
						if(val[6] == "GLOBAL") then val = nil end
						db.profile.SpellDB[spellid].OutputFlags[6] = val or nil
					else
						db.profile.OutputFlags[6] = val or nil
					end
				end,
				get = function()
					local v
					if(spellid) then
						if not db.profile.SpellDB[spellid].OutputFlags[6] then
							v = "!0_GLOBAL"
						else
							v = GetChannelForm(db.profile.SpellDB[spellid].OutputFlags[6])
						end
					else
						v = GetChannelForm(db.profile.OutputFlags[6])
					end
					return v
				end,
			},
		}
		return tbl
	end
	
	options = {
		name = 'WakeSpams',
		type = "group",
		args = {
			general = {
				name = 'WakeSpams',
				type = "group",
				args = {
					enabled = {
						order = 1,
						type = "toggle",
						name = "Enabled",
						desc = "Enable or disable addon.",
						descStyle = "inline",
						set = function(i,v)
							if addon:IsEnabled() then
								addon:Disable()
								db.profile.disabled = true
							else
								addon:Enable()
								db.profile.disabled = false
							end
						end,
						get = function() return addon:IsEnabled() end,
					},
					spellheader = {
						name = "Spell Announcements",
						type = "header",
						order = 10,
					},
					aoe = {
						order = 11,
						type = "toggle",
						name = "AoE",
						desc = "Enable or disable announcements for multitarget spells.",
						descStyle = "inline",
						set = function(v) db.profile.aoe = not db.profile.aoe end,
						get = function() return db.profile.aoe end,
					},
					interrupts = {
						order = 12,
						type = "toggle",
						name = "Interrupts",
						desc = "Enable or disable announcements for interrupts.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventDB["SPELL_INTERRUPT"] then
								db.profile.EventDB["SPELL_INTERRUPT"] = {}
							end
							db.profile.EventDB["SPELL_INTERRUPT"].disabled = not v
						end,
						get = function()
							if not db.profile.EventDB["SPELL_INTERRUPT"] then
								return true
							else
								return not db.profile.EventDB["SPELL_INTERRUPT"].disabled
							end
						end,
					},
					dispels = {
						order = 13,
						type = "toggle",
						name = "Dispels",
						desc = "Enable or disable announcements for successful dispels.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventDB["SPELL_DISPEL"] then
								db.profile.EventDB["SPELL_DISPEL"] = {}
							end
							db.profile.EventDB["SPELL_DISPEL"].disabled = not v
						end,
						get = function()
							if not db.profile.EventDB["SPELL_DISPEL"] then
								return true
							else
								return not db.profile.EventDB["SPELL_DISPEL"].disabled
							end
						end,
					},
					dispelresists = {
						order = 14,
						type = "toggle",
						name = "Dispel Resists",
						desc = "Enable or disable announcements for dispel resists.",
						descStyle = "inline",
						set = function(i,v)
							if not db.profile.EventDB["SPELL_DISPEL_FAILED"] then
								db.profile.EventDB["SPELL_DISPEL_FAILED"] = {}
							end
							db.profile.EventDB["SPELL_DISPEL_FAILED"].disabled = not v
						end,
						get = function()
							if not db.profile.EventDB["SPELL_DISPEL_FAILED"] then
								return true
							else
								return not db.profile.EventDB["SPELL_DISPEL_FAILED"].disabled
							end
						end,
					},
					defaults = {
						name = "Edit Defaults",
						type = "header",
						order = 20,
					},
					AbilityStart = {
						name = "",
						type = "group",
						inline = true,
						order = 21,
						args = {
							input = {
								order = 3,
								type = "input",
								name = "Default ability start message",
								desc = "Settings specific to spells override this.",
								width = "full",
								set = function(i,v) db.profile.AbilityStart = v end,
								get = function() return db.profile.AbilityStart end,
							},
						},
					},
					AbilityEnd = {
						name = "",
						type = "group",
						inline = true,
						order = 22,
						args = {
							input = {
								order = 3,
								type = "input",
								name = "Default ability end message",
								desc = "Settings specific to spells override this.",
								width = "full",
								set = function(i,v) db.profile.AbilityEnd = v end,
								get = function() return db.profile.AbilityEnd end,
							},
						},
					},
					strreplace = {
						name = "String replacements",
						type = "header",
						order = 30,
					},
					strreplacedesc = {
						name = helptext,
						type = "description",
						fontSize = "medium",
						order = 31,
					},
					output = {
						name = "",
						type = "group",
						order = 35,
						inline = true,
						args = SetupOutputOptions(),
					},
					resetheader = {
						name = "Reset To Defaults",
						type = "header",
						order = 40,
					},
					resetbtn = {
						name = "Reset all",
						type = "execute",
						order = 100,
						width = "half",
						confirm = function() return "Reset all WakeSpams settings to default? This also removes any custom spell settings you might have." end,
						func = function() db:ResetProfile() options.args.spells.args.edit = nil end,
					},
				},
			},
		},
	}
	
	local function EditSpell(spellid)
		local sname, _, sicon, _, _, _, _, _, _ = GetSpellInfo(spellid)
		local tbl = {
			name = "Edit Spell",
			type = "group",
			order = 1,
			args = {
				description = {
					name = db.profile.SpellDB[spellid].configname or sname,
					fontSize = "large",
					type = "description",
					order = 1,
					width = "full",
					image = sicon,
					imageCoords = {0.1,0.9,0.1,0.9},
					imageWidth = 20,
					imageHeight = 20,
				},
				desctwo = {
					name = "|cffffae00Spell ID: "..spellid.."|r",
					fontSize = "",
					type = "description",
					order = 2,
					width = "full",
				},
				spacerone = {
					name = "Spell Behaviour",
					type = "header",
					order = 3,
				},
				duration = {
					name = "Duration of the spell (in seconds)",
					desc = "Enter the duration of the spell here. The addon doesn't look up durations automatically, so if you have a talent that increases the duration, you will have to change it manually.",
					type = "range",
					min = 0,
					max = 1800,
					softMax = 40,
					step = 1,
					order = 30,
					set = function(info,val) db.profile.SpellDB[spellid].dur = val end,
					get = function() return db.profile.SpellDB[spellid].dur end,
				},
				aoe = {
					name = "AoE spell",
					desc = "Toggle this if it's a spell that hits multiple targets. Toggling this will also ignore custom start and end messages.",
					type = "toggle",
					order = 31,
					disabled = function() return (db.profile.SpellDB[spellid].custom) and nil or true end,
					set = function(info,val) db.profile.SpellDB[spellid].aoe = val end,
					get = function() return db.profile.SpellDB[spellid].aoe end,
				},
				countdown = {
					name = "Countdown in chat",
					desc = "Counts down in the chat when the spell is about to fade. Set to 0 for no countdown.",
					type = "range",
					min = 0,
					max = 5,
					step = 1,
					order = 32,
					set = function(info,val) db.profile.SpellDB[spellid].countdown = val end,
					get = function() return db.profile.SpellDB[spellid].countdown end,
				},
				spellfade = {
					name = "Announce when the spell fades",
					desc = "Toggle to announce when the spell fades.",
					type = "toggle",
					order = 33,
					set = function(info,val) db.profile.SpellDB[spellid].fade = val end,
					get = function() return db.profile.SpellDB[spellid].fade end,
				},
				spacertwo = {
					name = "Chat Customization",
					type = "header",
					order = 49,
				},
				txt = {
					name = "Start message of spell. Leave blank for default.",
					type = "input",
					order = 50,
					width = "full",
					disabled = function() return (db.profile.SpellDB[spellid].aoe) and true or nil end,
					set = function(info,val) db.profile.SpellDB[spellid].txt = val end,
					get = function() return db.profile.SpellDB[spellid].txt end,
				},
				txttwo = {
					name = "End message of spell. Leave blank for default.",
					type = "input",
					order = 51,
					width = "full",
					disabled = function() return (db.profile.SpellDB[spellid].aoe) and true or nil end,
					set = function(info,val) db.profile.SpellDB[spellid].txtend = val end,
					get = function() return db.profile.SpellDB[spellid].txtend end,
				},
				output = {
					name = "",
					type = "group",
					order = 52,
					inline = true,
					disabled = function() return (db.profile.SpellDB[spellid].whisper) and true or nil end,
					args = SetupOutputOptions(spellid),
				},
				buttonOkay = {
					name = "Okay",
					type = "execute",
					order = 100,
					width = "half",
					func = function()
						options.args.spells.args.edit = nil
					end,
				},
				buttonDefaults = {
					name = "Defaults",
					type = "execute",
					order = 101,
					width = "half",
					func = function() addon:ResetSpell(spellid) end,
				},
			},
		}
		tbl.args.output.args.whisper = {
			name = "Always whisper target",
			desc = "Toggle this if you wish to whisper the target of the spell.",
			type = "toggle",
			order = 10,
			disabled = function() return nil end,
			set = function(info,val) db.profile.SpellDB[spellid].whisper = val end,
			get = function() return db.profile.SpellDB[spellid].whisper end,
		}
		return tbl
	end
	
	--[[local function AddSpellDetails(spellid)
		local tbl
		local spell
		spellid = tonumber(spellid)
		if not addon:isNumeric(spellid) then -- Not a number
			tbl = {
				name = "Error: The entered Spell ID is not a number.",
				type = "description",
				order = 2,
				fontSize = "large",
			}
		elseif not GetSpellInfo(spellid) then -- Spell does not exist
			tbl = {
				name = ("Error: Could not find spell with the following Spell ID: %s"):format(spellid),
				type = "description",
				order = 2,
				fontSize = "large",
			}
		elseif ((module.SpellsDB[spellid]) or (db.profile.SpellDB[spellid])) then -- Already in the list
			spell = GetSpellInfo(spellid)
			tbl = {
				name = "",
				type = "group",
				inline = true,
				order = 2,
				args = {
					description = {
						name = ("Error: \"%s\" is already in the spell list. Do you wish to edit that spell instead?"):format(spell),
						type = "description",
						order = 1,
						fontSize = "large",
					},
					editbtn = {
						name = "Edit",
						type = "execute",
						order = 2,
						func = function()
							if(options.args.spells.args.oldlist) then
								options.args.spells.args.list = options.args.spells.args.oldlist
								options.args.spells.args.oldlist = nil
							else
								options.args.spells.args.oldlist = options.args.spells.args.list
								options.args.spells.args.list = nil
							end
							options.args.spells.args.edit = EditSpell(spellid)
						end,
					},
				},
			}
		else
			tbl = {
				name = "",
				type = "group",
				inline = true,
				order = 2,
				args = {
					description = {
						name = "Ok",
						type = "description",
						order = 1,
						fontSize = "large",
					},
				},
			}
		end
		return tbl
	end
	
	local function AddSpell()
		local spellid
		local tbl = {}
		tbl = {
			description = {
				name = "Add Spell",
				type = "group",
				inline = true,
				order = 1,
				args = {
					spellid = {
						name = "Spell ID. You can find Spell IDs for specific spells on WoWhead.",
						type = "input",
						order = 1,
						width = "full",
						get = function() return spellid or "Enter Spell ID #" end,
						set = function(i,v) spellid = v tbl.spellDetails = AddSpellDetails(spellid) return v end,
					},
				},
			},
			spellDetails = {
				name = "Enter a spell ID",
				type = "description",
				order = 2,
				fontSize = "large",
			},
		}
		return tbl
	end]]
	
	local function PopulateClassOptions()
		local tbl
		tbl = {
			name = 'spells',
			type = "group",
			desc = "Spells",
			childGroups = "tab",
			args = {
				--[[add = {
					name = "Add Spell",
					type = "group",
					desc = "Add spell",
					order = 3,
					args = AddSpell(),
				},]]
				list = {
					name = "Spell List",
					type = "group",
					desc = "Spell list",
					order = 2,
					args = {
					},
				},
			},
		}
		if not addon.DefaultSpellDB then
			local class, _ = UnitClass("player");
			tbl.args.list.args.info = {
				type = "description",
				name = ("There are no preconfigured %s spells. To add a spell, click the  \"Add Spell\" button."):format(class),
				order = 100,
			}
			return tbl
		end
		for i,v in pairs(addon.DefaultSpellDB) do
			local sname, _, sicon, _, _, _, _, _, _ = GetSpellInfo(i)
			local spell = addon.DefaultSpellDB[i]
			tbl.args.list.args[""..i..""] = {
				name = spell.configname or sname,
				type = "group",
				inline = true,
				order = i,
				args = {
					description = {
						name = "",
						type = "description",
						order = 1,
						image = sicon,
						width = "half",
						imageCoords = {0.1,0.9,0.1,0.9},
						imageWidth = 18,
						imageHeight = 18,
					},
					toggleSpell = {
						name = "Enable",
						type = "toggle",
						order = 3,
						width = "half",
						desc = GetSpellLink(i)..' Enable/disable announcing for this spell.',
						--descStyle = "inline",
						set = function(info,val)
							if not db.profile.SpellDB[i] then
								db.profile.SpellDB[i] = {}
							end
							db.profile.SpellDB[i].disabled = not val
						end,
						get = function()
							if not db.profile.SpellDB[i] then
								return true
							else
								return not db.profile.SpellDB[i].disabled
							end
						end,
					},
					editSpell = {
						name = "Edit",
						type = "execute",
						order = 2,
						width = "half",
						desc = GetSpellLink(i)..' Edit this spell.',
						func = function()
							if not db.profile.SpellDB[i] then
								db.profile.SpellDB[i] = spell
							end
							if(options.args.spells.args.oldlist) then
								options.args.spells.args.list = options.args.spells.args.oldlist
								options.args.spells.args.oldlist = nil
							else
								options.args.spells.args.oldlist = options.args.spells.args.list
								options.args.spells.args.list = nil
							end
							options.args.spells.args.edit = EditSpell(i)
						end,
					}
				}
			}
		end
		return tbl
	end
	options.args.spells = PopulateClassOptions()
	
	options.args.help = {
		type = "group",
		name = "Help",
		args = {
			strreplace = {
				name = "String replacements",
				type = "header",
				order = 1,
			},
			strreplacedesc = {
				name = helptext,
				type = "description",
				fontSize = "medium",
				order = 2,
			},
		},
	}
	
	options.args.profiles = LibStub("AceDBOptions-3.0"):GetOptionsTable(db)
end

function module:ShowConfig()
	InterfaceOptionsFrame_OpenToCategory(self.InterfaceOptions)
end

function module:OnInitialize()
	db = addon.db
	self:RegisterOptions()
	ACR:RegisterOptionsTable("WakeSpams", options)
	
	self.InterfaceOptions = ACD:AddToBlizOptions("WakeSpams", "WakeSpams", nil, "general")
	local class, _ = UnitClass("player");
	ACD:AddToBlizOptions("WakeSpams", class, "WakeSpams", "spells")
	ACD:AddToBlizOptions("WakeSpams", "Help", "WakeSpams", "help")
	ACD:AddToBlizOptions("WakeSpams", "Profiles", "WakeSpams", "profiles")
	LibStub("LibAboutPanel").new("WakeSpams", "WakeSpams")
end